# ITHIM Population Projections 2010-2050 
# Pop_Projections_20180731.R Sarah Grajdura 7/31/2018

# R program that reads 2010-2050 age-sex populations by county
# from Dept. of Finance (DOF) Table P-2 (edited: deleted rows (1:3, 2557) ) 
# http://www.dof.ca.gov/Forecasting/Demographics/Projections/
# and aggregates ages in 5-year average increments to ITHIM age categories
# at the county, MPO, region, and state levels 

# Set working directory 
directory <- "/Users/sgrajdu2/Desktop/Population Projections"
setwd(directory)

# Load needed packages
#install.packages("stringr")
#install.packages("dplyr")
#install.packages("tidyverse")
library(stringr)
library(dplyr)
library(tidyverse)

# Load data
# DOF California population projections
ca_pop <- read.csv("P2_Age_Group_Sex_1yr_Nosup_interim_edited.csv", sep = ",", header = TRUE) 

# Key to California counties, MPO's, & regions 
ca_geographies <- read.csv("CA_County_FIPS2Region2010.csv", sep = ",", header = TRUE)




# STEP 1. Aggregate to ITHIM age groups (county level)

# Make new variable to represent ITHIM age groups,  
# a repeating string of age intervals assigned to 
# each of the 22 unique existing ca_pop_proj$Age.group values
ca_pop$age_ithim <- as.factor(rep(c(rep("0-4", 2), rep("5-14", 2), rep("15-29", 3), rep("30-44", 3), rep("45-59", 3), rep("60-69", 2), rep("70-79", 2), rep("80+", 5)), (nrow(ca_pop)/22)))

# Order the age interval factor variable 
ca_pop$age_ithim <- factor(ca_pop$age_ithim , levels = c("0-4", "5-14", "15-29", "30-44", "45-59", "60-69", "70-79", "80+"))

# Aggregate population columns 4:54 of ca_pop 
# by county, age_ithim, and gender 
county_pop <- aggregate(ca_pop[4:54], by=list(ca_pop$County, ca_pop$age_ithim, ca_pop$Sex), FUN = sum)

# Rename first 3 columns of county_pop
colnames(county_pop)[1:3] <- c("County", "Age_Group", "Sex")




# STEP 2. Average over 5-year intervals (county level)

# Create new variables for 5-year population intervals
# that are the populaton averages for successive 5yr intervals
county_pop$avg2015_2019 <- apply(county_pop[9:13], 1, mean) #get mean of columns 9 to 13 for all rows
county_pop$avg2020_2024 <- apply(county_pop[14:18], 1, mean)
county_pop$avg2025_2029 <- apply(county_pop[19:23], 1, mean)
county_pop$avg2030_2034 <- apply(county_pop[24:28], 1, mean)
county_pop$avg2035_2039 <- apply(county_pop[29:33], 1, mean)
county_pop$avg2040_2044 <- apply(county_pop[34:38], 1, mean)
county_pop$avg2045_2049 <- apply(county_pop[39:43], 1, mean)
county_pop$avg2050_2054 <- apply(county_pop[44:48], 1, mean)

# Simplify dataframe with just the information of interest
county_pop <- county_pop[c(1:3, 55:62)]

# Sort by county
county_pop <- county_pop[order(county_pop$County),]

# Save county file (wide format) if desired
#write.csv(county_pop, "/Users/sgrajdu2/Desktop/Population Projections/Output data/county_wide_5yrpop20180731.csv")

# Reshape data to long format and sort by county
county_pop_long <- gather(county_pop, year, pop, avg2015_2019:avg2050_2054)
county_pop_long <- county_pop_long[order(county_pop_long$County),]

# Save count file (long format) if desired
write.csv(county_pop_long, "/Users/sgrajdu2/Desktop/Population Projections/Output data/county_pop_5yrpop20180731.csv")





# STEP 3. Aggregate counties to MPO-level (county to MPO level)

# Merge MPO info to Counties using the long version of county_pop
# but first remove the word "County" from county_pop$County by splitting
# at the word "County" but since that's the last word
# for all counties, we are left with County names alone in tmp$V1
tmp <- as.data.frame(str_split_fixed(county_pop_long$County, " County", 2))

# Make new variable & assign county names without "County" to new var county_pop$County
county_pop_long$County <- tmp$V1

# Rename ca_geographies "countyname" to "County" for merge to work
ca_geographies$County <- ca_geographies$countyname

# Finally merge county_pop with ca_geographies by id=County
county_pop_long_FIPS <- merge.data.frame(county_pop_long, ca_geographies)

# Aggregate to the MPO level over age group, sex, and year
mpo_pop <- aggregate(county_pop_long_FIPS$pop, by=list(county_pop_long_FIPS$Age_Group, county_pop_long_FIPS$Sex, county_pop_long_FIPS$mpo_name, county_pop_long_FIPS$year), FUN = sum)

# Change column names 
colnames(mpo_pop) <- c("Age_Group", "Sex", "MPO", "Year", "Pop")

# Save aggregated MPO populations
write.csv(mpo_pop, "/Users/sgrajdu2/Desktop/Population Projections/Output data/mpo_pop_5yrpop20180731.csv")





# STEP 4. Aggregate counties to region

# Go back and use the county pop data with FIPS codes
# to aggregate to Region instead of MPO over age, sex, and year
region_pop <- aggregate(county_pop_long_FIPS$pop, by=list(county_pop_long_FIPS$Age_Group, county_pop_long_FIPS$Sex, county_pop_long_FIPS$region_name, county_pop_long_FIPS$year), FUN = sum)

# Change column names 
colnames(region_pop) <- c("Age_Group", "Sex", "Region", "Year", "Pop")

# Save aggregated Region populations
write.csv(region_pop, "/Users/sgrajdu2/Desktop/Population Projections/Output data/region_pop_5yrpop20180731.csv")





# STEP 6. Aggregate counties to state totals 

# Go back and use the county pop data with FIPS codes
# to aggregate over the age, sex, and year to get state totals
state_pop <- aggregate(county_pop_long_FIPS$pop, by=list(county_pop_long_FIPS$Age_Group, county_pop_long_FIPS$Sex, county_pop_long_FIPS$year), FUN = sum)

# Change column names 
colnames(state_pop) <- c("Age_Group", "Sex", "Year", "Pop")

# Save aggregated State populations
write.csv(state_pop, "/Users/sgrajdu2/Desktop/Population Projections/Output data/state_pop_5yrpop20180731.csv")




# STEP 7. Create the full dataframe 

# The order of columns will be Geo_Unit (county, MPO, region, state), 
# Unit_Name (county name, MPO name, etc.), Age_Group, Sex, Year, Population. 
# In order to append the county, MPO, region, and state dataframes, they 
# must each have the same columns in the same order. 

# 1. Reformat county dataframe 
tmp <- rep("County", nrow(county_pop_long)) #make column vector for "Geo_Unit" variable
county_pop_long <- cbind(tmp, county_pop_long)
colnames(county_pop_long) <- c("Geo_Unit", "Unit_Name", "Age_Group", "Sex", "Year", "Pop")

# 2. Reformat MPO dataframe
tmp <- as.data.frame(rep("MPO", nrow(mpo_pop))) #make column vector for "Geo_Unit" variable
tmp2 <- as.data.frame(mpo_pop$MPO) #make column vector for "Unit_Name"
tmp3 <- cbind(tmp, tmp2)
colnames(tmp3) <- c("Geo_Unit", "Unit_Name") 
mpo_pop <- subset(mpo_pop, select = -MPO ) #remove the "MPO" column
mpo_pop <- cbind(tmp3, mpo_pop)

# 3. Reformat Region dataframe
tmp <- as.data.frame(rep("Region", nrow(region_pop)))
tmp2 <- as.data.frame(region_pop$Region)
tmp3 <- cbind(tmp, tmp2)
colnames(tmp3) <- c("Geo_Unit", "Unit_Name") 
region_pop <- subset(region_pop, select = -Region) #remove Region column
region_pop <- cbind(tmp3, region_pop)

# 4. Reformat State dataframe
tmp <- as.data.frame(rep("State", nrow(state_pop)))
tmp2 <- as.data.frame(rep("California", nrow(state_pop)))
tmp3 <- cbind(tmp, tmp2)
colnames(tmp3) <- c("Geo_Unit", "Unit_Name") 
state_pop <- cbind(tmp3, state_pop)

# 5. Append the dataframes to make final full dataframe 
full_pop <- rbind(county_pop_long, mpo_pop)
full_pop <- rbind(full_pop, region_pop)
full_pop <- rbind(full_pop, state_pop) #final full dataframe

write.csv(full_pop, "/Users/sgrajdu2/Desktop/Population Projections/Output data/full_pop_5yrpop20180731.csv")
